local ADDON_NAME, addon = ...

local cfgreg = LibStub("AceConfigRegistry-3.0")
local cfgdialog = LibStub('AceConfigDialog-3.0')

local statusbars = LibStub("LibSharedMedia-3.0"):List("statusbar")
local fonts = LibStub("LibSharedMedia-3.0"):List("font")

local L = setmetatable({}, { __index = function(t, k)
	local v = tostring(k)
	rawset(t, k, v)
	return v
end })

addon.L = L

--######################
--	Groups
--######################

local setupopts = {
	order = 0,
	type = "group",
	name = L["Groups"],
	args = {},
}

function addon:AddGroup(group, new)
	local db

	if(new) then
		local label = group.name
		addon.db.profile.groups[label] = {}
		db = addon.db.profile.groups[label]

		db.label = label
		db.type = group.type
		db.enabled = true
		db.style = "default"
		db.visible = {raid = {}}
		db.auras = ""
		db.scale = 1
	else
		db = addon.db.profile.groups[group.label]
	end

	setupopts.args[db.label] = {
		order = 4,
		type = "group",
		inline = false,
		name = db.label,
		args = {
			enabled = {
				order = 0,
				type = "toggle",
				name = L["Enable"],
				set = function(info, val)
					db.enabled = val
					addon:LibreApply()
				end,
				get = function(info) return db.enabled end,
				validate = function(info, val)
					if(not InCombatLockdown()) then
						return true
					else
						return L["In Combat"]
					end
				end,
			},

			delete = {
				order = 1,
				type = "execute",
				name = L["Delete"],
				--width = "half",
				func = function(info)	
					db.kill = true
					addon:LibreApply()
					setupopts.args[db.label] = nil
				end,
				confirm = true,
				confirmText = L["Kill it with fire?"],
			},

			spacer = {
				order = 2,
				type = "description",
				name = "",
			},

			style = {
				order = 3,
				type = "select",
				name = L["Style"],
				set = function(info, val)
					db.style = val
					addon:LibreApply()
				end,
				get = function(info)
					return db.style
				end,
				values = function(info)
					local styles = {}
					for k, v in next, addon.db.global.styles do
						styles[k] = k
					end

					return styles
				end,
			},

			auras = {
				order = 4,
				type = "select",
				name = L["Aura List"],
				set = function(info, val)
					db.auras = val
					addon:LibreApply()
				end,
				get = function(info)
					return db.auras
				end,
				values = function(info)
					local auraLists = { [""] = L["None"], }
					for k, v in next, addon.db.global.auraLists do
						auraLists[k] = k
					end

					return auraLists
				end,
			},

			visibility = {
				order = 5,
				type = "group",
				inline = true,
				name = L["Visibility"],
				args = {
					solo = {
						order = 1,
						type = "toggle",
						name = L["Solo"],
						get = function(info) return db.visible.solo end,
						set = function(info, val) db.visible.solo = val and true or nil; addon:LibreApply() end,
					},
					party = {
						order = 2,
						type = "toggle",
						name = L["Party"],
						get = function(info) return db.visible.party end,
						set = function(info, val) db.visible.party = val and true or nil; addon:LibreApply() end,
					},
					raidgroup = {
						order = 3,
						type = "group",
						name = L["Raid"],
						args = {
							raid = {
								order = 3,
								type = "toggle",
								name = L["Raid"],
								get = function(info) return db.visible.raid.raid end,
								set = function(info, val)
									if(val) then
										db.visible.raid = {raid = true}
									end

									addon:LibreApply()
								end,
							},
							raid10 = {
								order = 4,
								type = "toggle",
								name = L["Raid 10"],
								get = function(info) return db.visible.raid.raid10 end,
								set = function(info, val)
									if(val) then
										db.visible.raid = {raid10 = true}
									end

									addon:LibreApply()
								end,
							},
							raid1025 = {
								order = 5,
								type = "toggle",
								name = L["Raid 10 / 25"],
								get = function(info) return db.visible.raid.raid1025 end,
								set = function(info, val)
									if(val) then
										db.visible.raid = {raid1025 = true}
									end

									addon:LibreApply()
								end,
							},
							raid25 = {
								order = 6,
								type = "toggle",
								name = L["Raid 25"],
								get = function(info) return db.visible.raid.raid25 end,
								set = function(info, val)
									if(val) then
										db.visible.raid = {raid25 = true}
									end

									addon:LibreApply()
								end,
							},
							raid2540 = {
								order = 7,
								type = "toggle",
								name = L["Raid 25 / 40"],
								get = function(info) return db.visible.raid.raid2540 end,
								set = function(info, val)
									if(val) then
										db.visible.raid = {raid2540 = true}
									end

									addon:LibreApply()
								end,
							},
							raid40 = {
								order = 8,
								type = "toggle",
								name = L["Raid 40"],
								get = function(info) return db.visible.raid.raid40 end,
								set = function(info, val)
									if(val) then
										db.visible.raid = {raid40 = true}
									end

									addon:LibreApply()
								end,
							},
						},
					},
				},
			},

			options = {
				order = 6,
				type = "group",
				name = L["Options"],
				inline = true,
				args ={
					unitsPer = {
						order = 2,
						type = "range",
						name = L["Units Per Column"],
						step = 1,
						min = 1,
						max = 40,
						disabled = function(info) return db.keeptogether end,
						get = function(info) return db.unitsPer end,
						set = function(info, val) db.unitsPer = val; addon:LibreApply() end,
					},

					maxCol = {
						order = 3,
						type = "range",
						name = L["Max Columns"],
						step = 1,
						min = 1,
						max = 40,
						disabled = function(info) return db.keeptogether end,
						get = function(info) return db.maxCol end,
						set = function(info, val) db.maxCol = val; addon:LibreApply() end,
					},

					scale = {
						order = 4,
						type = "range",
						name = L["Scale"],
						step = 0.05,
						min = 0.5,
						max = 2,
						get = function(info) return db.scale end,
						set = function(info, val) db.scale = val; addon:LibreApply() end,
					},	
				},
			},
		},
	}

	if(db.type == "group") then
		setupopts.args[db.label].args.options.args.keeptogether = {
			order = 0,
			type = "toggle",
			name = L["Keep Groups Together"],
			get = function(info) return db.keeptogether end,
			set = function(info, val) db.keeptogether = val; addon:LibreApply() end,
		}

		setupopts.args[db.label].args.options.args.spacer = {
			order = 1,
			type = "description",
			name = "",
		}
	end

	return db
end

function addon:CreateGroups()
	for k, v in next, addon.db.profile.groups do
		addon:AddGroup(v)

		local header
		for x, t in next, addon.headers[addon.profile] do
			if(k == t.db.label) then
				header = t
				break
			end
		end

		if(not header and not InCombatLockdown()) then
			addon:CreateHeader(v)
		end
	end
end

local groupTypes = {
	["group"] = L["Group"],
	["pets"] = L["Pets"],
}

local createGroup = {}
setupopts.args["create"] = {
	order = 2,
	type = "group",
	inline = true,
	name = "", 
	get = function(info) return createGroup[info[#(info)]] end,
	args = {
		name = {
			order = 0,
			type = "input",
			name = L["New Group"],
			set = function(info, val)
				val = val:trim()
				createGroup[info[#(info)]] = val ~= "" and val
			end,
			get = function(info) return createGroup.name end,
			validate = function(info, val)
				val = val:trim()
				if(not addon.db.profile.groups[val]) then
					return true
				else
					return L["Name: Already Exists"]
				end
			end,
		},

		type = {
			order = 1,
			type = "select",
			name = L["Type"],
			set = function(info, val) 
				createGroup[info[#(info)]] = val
			end,
			values = groupTypes,
		},

		create = {
			order = 2,
			type = "execute",
			name = L["Create"],
			disabled = function(info)
				return (not createGroup.name or not createGroup.type)
			end,
			func = function(info)
				local group = addon:AddGroup(createGroup, true)

				addon:CreateHeader(group)
				createGroup.name = nil
				createGroup.type = nil
			end,
		},
	},
}

--######################
--	Styles
--######################

local fontflags = {
	[""] = L["None"],
	["OUTLINE"] = L["Outline"],
}

local optStyle = {style = "default"}
local styleopts = {
	order = 1,
	type = "group",
	name = L["Styles"],
	args = {
		stylegroup = {
			order = 0,
			type = "group",
			name = "",
			inline = true,
			args = {
				name = {
					order = 0,
					type = "input",
					name = L["New Style"],
					set = function(info, val)
						val = val:trim()

						if(val ~= "") then
							addon.db.global.styles[val] = {}
							addon:UpdateStyles()
						end
					end,
					validate = function(info, val)
						val = val:trim()
						if(not addon.db.global.styles[val]) then
							return true
						else
							return L["Name: Already Exists"]
						end
					end,
				},	

				style = {
					order = 2,
					type = "select",
					name = L["Configure Style"],
					set = function(info, val)	
						optStyle[info[#info]] = val
					end,
					get = function(info) return optStyle[info[#info]] end,
					values = function(info)
						local styles = {}
						for k, v in next, addon.db.global.styles do
							styles[k] = k
						end

						return styles
					end,
				},	

				delete = {
					order = 3,
					type = "select",
					name = L["Delete Style"],
					set = function(info, val)	
						for k, v in next, addon.headers do
							for x, t in next, v do	
								if(t.db.style == val) then
									t.db.style = "default"
								end
							end
						end

						addon:LibreApply()
						addon.db.global.styles[val] = nil
						optStyle.style = "default"
					end,		
					values = function(info)
						local styles = {}
						for k, v in next, addon.db.global.styles do
							if(k ~= "default") then
								styles[k] = k
							end
						end

						return styles
					end,
					confirm = true,
					confirmText = L["Kill it with fire?"],
				},
			},
		},
	},
}

styleopts.args["Size"] = {
	order = 5,
	type = "group",
	inline = false,
	name = L["Size"],
	args = {
		width = {
			order = 0,
			type = "range",
			name = L["Width"],
			step = 1,
			min = 30,
			max = 300,
			get = function(info) return addon.db.global.styles[optStyle.style].size.w end,
			set = function(info, val)
				addon.db.global.styles[optStyle.style].size.w = val
				addon:LibreApply()
			end,
		},

		height = {
			order = 1,
			type = "range",
			name = L["Height"],
			step = 1,
			min = 30,
			max = 300,
			get = function(info) return addon.db.global.styles[optStyle.style].size.h end,
			set = function(info, val)
				addon.db.global.styles[optStyle.style].size.h = val
				addon:LibreApply()
			end,
		},	
	},
}

styleopts.args["Health"] = {
	order = 6,
	type = "group",
	inline = false,
	name = L["Health Bar"],
	args = {
		statusbar = {
			order = 1,
			type = "select",
			name = L["StatusBar"],
			get = function(info)
				for k, v in next, statusbars do
					if(v == addon.db.global.styles[optStyle.style].health.statusbar) then return k end
				end
			end,
			set = function(info, val)
				addon.db.global.styles[optStyle.style].health.statusbar = statusbars[val]
				addon:LibreApply()
			end,
			values = statusbars,
			itemControl = "DDI-Statusbar",
		},
	},
}

styleopts.args["Power"] = {
	order = 7,
	type = "group",
	inline = false,
	name = L["Power Bar"],
	args = {
		statusbar = {
			order = 1,
			type = "select",
			name = L["StatusBar"],
			get = function(info)
				for k, v in next, statusbars do
					if(v == addon.db.global.styles[optStyle.style].power.statusbar) then return k end
				end
			end,
			set = function(info, val)
				addon.db.global.styles[optStyle.style].power.statusbar = statusbars[val]
				addon:LibreApply()
			end,
			values = statusbars,
			itemControl = "DDI-Statusbar",
		},
	},
}

styleopts.args["Name"] = {
	order = 8,
	type = "group",
	inline = false,
	name = L["Name"],
	args = {
		text = {
			order = 1,
			type = "group",
			inline = true,
			name = L["Text"],
			args = {
				font = {
					order = 1,
					type = "select",
					name = L["Font"],
					get = function(info)
						for k, v in next, fonts do
							if(v == addon.db.global.styles[optStyle.style].name.font) then return k end
						end
					end,
					set = function(info, val)
						addon.db.global.styles[optStyle.style].name.font = fonts[val]
						addon:LibreApply()
					end,
					values = fonts,
					itemControl = "DDI-Font",
				},

				fontflag = {
					order = 2,
					type = "select",
					name = L["Font Flag"],
					get = function(info)
						return addon.db.global.styles[optStyle.style].name.fontflag
					end,
					set = function(info, val)
						addon.db.global.styles[optStyle.style].name.fontflag = val
						addon:LibreApply()
					end,
					values = function(info)
						return fontflags
					end,
				},

				fontsize = {
					order = 3,
					type = "range",
					name = L["Font Size"],
					min = 6,
					max = 32,
					step = 1,
					get = function(info)
						return addon.db.global.styles[optStyle.style].name.fontsize
					end,
					set = function(info, val)
						addon.db.global.styles[optStyle.style].name.fontsize = val
						addon:LibreApply()
					end,
				},
			},
		},
	},
}

styleopts.args["Status"] = {
	order = 9,
	type = "group",
	inline = false,
	name = L["Status"],
	args = {
		text = {
			order = 1,
			type = "group",
			inline = true,
			name = L["Text"],
			args = {
				font = {
					order = 1,
					type = "select",
					name = L["Font"],
					get = function(info)
						for k, v in next, fonts do
							if(v == addon.db.global.styles[optStyle.style].status.font) then return k end
						end
					end,
					set = function(info, val)
						addon.db.global.styles[optStyle.style].status.font = fonts[val]
						addon:LibreApply()
					end,
					values = fonts,
					itemControl = "DDI-Font",
				},

				fontflag = {
					order = 2,
					type = "select",
					name = L["Font Flag"],
					get = function(info)
						return addon.db.global.styles[optStyle.style].status.fontflag
					end,
					set = function(info, val)
						addon.db.global.styles[optStyle.style].status.fontflag = val
						addon:LibreApply()
					end,
					values = function(info)
						return fontflags
					end,
				},

				fontsize = {
					order = 3,
					type = "range",
					name = L["Font Size"],
					min = 6,
					max = 32,
					step = 1,
					get = function(info)
						return addon.db.global.styles[optStyle.style].status.fontsize
					end,
					set = function(info, val)
						addon.db.global.styles[optStyle.style].status.fontsize = val
						addon:LibreApply()
					end,
				},

				color = {
					order = 4,
					type = "color",
					name = L["Color"],
					get = function(info)
						return unpack(addon.db.global.styles[optStyle.style].status.color)
					end,
					set = function(info,r,g,b)
						addon.db.global.styles[optStyle.style].status.color = {r,g,b}
						addon:LibreApply()
					end,
				},
			},
		},
	},
}

styleopts.args["Cluster"] = {
	order = 10,
	type = "group",
	inline = false,
	name = L["Cluster"],
	args = {
		text = {
			order = 1,
			type = "group",
			inline = true,
			name = L["Text"],
			args = {
				font = {
					order = 1,
					type = "select",
					name = L["Font"],
					get = function(info)
						for k, v in next, fonts do
							if(v == addon.db.global.styles[optStyle.style].cluster.font) then return k end
						end
					end,
					set = function(info, val)
						addon.db.global.styles[optStyle.style].cluster.font = fonts[val]
						addon:LibreApply()
					end,
					values = fonts,
					itemControl = "DDI-Font",
				},

				fontflag = {
					order = 2,
					type = "select",
					name = L["Font Flag"],
					get = function(info)
						return addon.db.global.styles[optStyle.style].cluster.fontflag
					end,
					set = function(info, val)
						addon.db.global.styles[optStyle.style].cluster.fontflag = val
						addon:LibreApply()
					end,
					values = function(info)
						return fontflags
					end,
				},

				fontsize = {
					order = 3,
					type = "range",
					name = L["Font Size"],
					min = 6,
					max = 32,
					step = 1,
					get = function(info)
						return addon.db.global.styles[optStyle.style].cluster.fontsize
					end,
					set = function(info, val)
						addon.db.global.styles[optStyle.style].cluster.fontsize = val
						addon:LibreApply()
					end,
				},

				color = {
					order = 4,
					type = "color",
					name = L["Color"],
					get = function(info)
						return unpack(addon.db.global.styles[optStyle.style].cluster.color)
					end,
					set = function(info,r,g,b)
						addon.db.global.styles[optStyle.style].cluster.color = {r,g,b}
						addon:LibreApply()
					end,
				},
			},
		},
	},
}

styleopts.args["Auras"] = {
	order = 11,
	type = "group",
	inline = false,
	name = L["Auras"],
	args = {
		indys = {
			order = 1,
			type = "group",
			inline = true,
			name = L["Indicators"],
			args = {
				size = {
					order = 1,
					type = "range",
					name = L["Size"],
					min = 2,
					max = 24,
					step = 1,
					get = function(info)
						return addon.db.global.styles[optStyle.style].auras.indys.size
					end,
					set = function(info, val)
						addon.db.global.styles[optStyle.style].auras.indys.size = val
						addon:LibreApply()
					end,
				},
				max = {
					order = 2,
					type = "range",
					name = L["Max Number"],
					min = 1,
					max = 10,
					step = 1,
					get = function(info)
						return addon.db.global.styles[optStyle.style].auras.indys.num
					end,
					set = function(info, val)
						addon.db.global.styles[optStyle.style].auras.indys.num = val
						addon:LibreApply()
					end,
				},
			},
		},

		icons = {
			order = 2,
			type = "group",
			inline = true,
			name = L["Icons"],
			args = {
				size = {
					order = 1,
					type = "range",
					name = L["Size"],
					min = 6,
					max = 32,
					step = 1,
					get = function(info)
						return addon.db.global.styles[optStyle.style].auras.icons.size
					end,
					set = function(info, val)
						addon.db.global.styles[optStyle.style].auras.icons.size = val
						addon:LibreApply()
					end,
				},
				bossSize = {
					order = 2,
					type = "range",
					name = L["Boss Debuff Size"],
					min = 6,
					max = 32,
					step = 1,
					get = function(info)
						return addon.db.global.styles[optStyle.style].auras.icons.bossSize
					end,
					set = function(info, val)
						addon.db.global.styles[optStyle.style].auras.icons.bossSize = val
						addon:LibreApply()
					end,
				},
				max = {
					order = 3,
					type = "range",
					name = L["Max Number"],
					min = 1,
					max = 10,
					step = 1,
					get = function(info)
						return addon.db.global.styles[optStyle.style].auras.icons.num
					end,
					set = function(info, val)
						addon.db.global.styles[optStyle.style].auras.icons.num = val
						addon:LibreApply()
					end,
				},
			},
		},
	},
}

--######################
--	Auras
--######################

local optAuras = {}
local auraopts = {
	order = 2,
	type = "group",
	name = L["Auras"],
	args = {
		auragroup = {
			order = 0,
			type = "group",
			name = "",
			inline = true,
			args = {
				name = {
					order = 1,
					type = "input",
					name = L["New Aura List"],
					set = function(info, val)
						val = val:trim()

						if(val ~= "") then
							optAuras[info[#info]] = val
						else
							optAuras[info[#info]] = nil
						end
					end,
					get = function(info) return optAuras[info[#info]] end,
					validate = function(info, val)
						val = val:trim()
						if(not addon.db.global.auraLists[val]) then
							return true
						else
							return L["AuraList: Already Exists"]
						end
					end,
				},

				copy = {
					order = 2,
					type = "select",
					name = L["Copy Aura List"],
					set = function(info, val)	
						optAuras[info[#info]] = val
					end,
					get = function(info) return optAuras[info[#info]] end,
					values = function(info)
						local list = {[""] = ""}
						for k, v in next, addon.db.global.auraLists do
							list[k] = k
						end

						return list
					end,
				},

				create = {
					order = 3,
					type = "execute",
					name = L["Create"],
					disabled = function(info) return not optAuras.name end,
					func = function(info)
						local list = optAuras.name
						local copy = optAuras.copy

						if(copy and copy ~= "") then
							addon:AddAuraList(list, true, copy)
						else
							addon:AddAuraList(list, true)
						end

						optAuras.name = nil
						optAuras.copy = nil

						addon:UpdateAuras()
					end,
				},
			},
		},
	},
}

function addon:AddAura(label, db, aura)
	local name, _, icon = addon.GetSpellInfo(aura)
	if(not name) then
		name = aura
	end

	if(not db[name]) then
		local spellid = tonumber(aura)

		db[name] = {	
			icons = false,	
			indicators = false,
			disableCD = false,
			color = {1,1,1},
			player = false,
			spellid = spellid,
		}
	end

	if(type(db[name] == "table")) then
		local spellid = db[name].spellid
		local spell = addon.GetSpellInfo(spellid)

		if(name == spell) then
			icon = GetSpellTexture(spellid)
		end
	end

	auraopts.args[label].args[name] = {
		order = 3,
		type = "group",
		inline = false,
		name = name,
		args = {
			name = {
				order = 1,
				type = "description",
				name = name,
				fontSize = "large",
				image = function() return icon, 24, 24 end,	
			},

			icon = {
				order = 2,
				type = "group",
				name = L["Icon"],
				inline = true,
				args = {
					enabled = {
						order = 1,
						type = "toggle",
						name = L["Enable"],
						get = function(info) return db[name].icons end,
						set = function(info, val) db[name].icons = val; addon:LibreApply() end,
					},
				},
			},

			indicator = {
				order = 3,
				type = "group",
				name = L["Indicator"],
				inline = true,
				args = {
					enabled = {
						order = 1,
						type = "toggle",
						name = L["Enable"],
						get = function(info) return db[name].indicators end,
						set = function(info, val) db[name].indicators = val; addon:LibreApply() end,
					},

					color = {
						order = 2,
						type = "color",
						name = L["Color"],
						get = function(info) return unpack(db[name].color) end,
						set = function(info,r,g,b) db[name].color = {r,g,b}; addon:LibreApply() end,
					},

					disableCD = {
						order = 3,
						type = "toggle",
						name = L["Disable Cooldown"],
						get = function(info) return db[name].disableCD end,
						set = function(info, val) db[name].disableCD = val; addon:LibreApply() end,
					},
				},
			},

			spellid = {
				order = 4,
				type = "input",
				name = L["Spell ID"],
				width = "half",
				get = function(info) return tostring(db[name].spellid) end,
				set = function(info, val) db[name].spellid = tonumber(val); addon:LibreApply() end,
			},

			player = {
				order = 6,
				type = "toggle",
				name = L["Is Mine"],
				get = function(info) return db[name].player end,
				set = function(info, val) db[name].player = val; addon:LibreApply() end,
			},

			spacer = {
				order = 7,
				type = "description",
				name = "",
			},

			delete = {
				order = 8,
				type = "execute",
				name = L["Delete"],
				--width = "half",
				func = function(info)
					db[name] = nil
					auraopts.args[label].args[name] = nil
				end,
				confirm = true,
				confirmText = L["Kill it with fire?"],
			},
		},
	}
end

function addon:AddAuraList(list, new, copy)
	local db

	if(new) then
		if(copy) then
			addon.db.global.auraLists[list] =  addon.deepcopy(addon.db.global.auraLists[copy])
			db = addon.db.global.auraLists[list]

			db.label = list
		else
			addon.db.global.auraLists[list] = {}
			db = addon.db.global.auraLists[list]

			db.label = list
			db.Raid_Buffs = addon.deepcopy(addon.RaidBuffs)
			db.Survival_CDs = addon.deepcopy(addon.SurvivalCooldowns)
			db.Boss_Debuffs = {}
			db.auras = {}
		end
	else
		db = addon.db.global.auraLists[list]
	end

	auraopts.args[db.label] = {
		order = 3,
		type = "group",
		inline = false,
		name = db.label,
		args = {
			new = {
				order = 1,
				type = "group",
				inline = true,
				name = L["New Aura"],
				args = { 
					aura = {
						order = 0,
						type = "input",
						name = L["Name or Spell ID"],
						set = function(info, val)
							addon:AddAura(db.label, db.auras, val)
						end,
						validate = function(info, val)
							local name = addon.GetSpellInfo(val)
							if(db.auras[val] or (name and db.auras[name])) then
								return L["Aura: Already Exists"]
							elseif(val ~= "") then
								return true
							end
						end,
					},
				},
			},

			dispel = {
				order = 2,
				type = "group",
				inline = true,
				name = L["Show Dispellable"],
				args = {
					Magic = {
						order = 1,
						type = "toggle",
						name = L["Magic"],
						get = function(info) return db["Magic"] end,
						set = function(info, val) db["Magic"] = val and true or nil end,
					},

					Curse = {
						order = 2,
						type = "toggle",
						name = L["Curse"],
						get = function(info) return db["Curse"] end,
						set = function(info, val) db["Curse"] = val and true or nil end,
					},

					Poison = {
						order = 3,
						type = "toggle",
						name = L["Poison"],
						get = function(info) return db["Poison"] end,
						set = function(info, val) db["Poison"] = val and true or nil end,
					},

					Disease = {
						order = 4,
						type = "toggle",
						name = L["Disease"],
						get = function(info) return db["Disease"] end,
						set = function(info, val) db["Disease"] = val and true or nil end,
					},
				},
			},

			delete = {
				order = 3,
				type = "execute",
				name = L["Delete"],
				--width = "half",
				func = function(info)	
					for k, v in next, addon.headers do
						for x, t in next, v do	
							if(t.db.auras == db.label) then
								t.db.auras = ""
							end
						end
					end

					addon:LibreApply()
					auraopts.args[db.label] = nil
					addon.db.global.auraLists[db.label] = nil
				end,
				confirm = true,
				confirmText = L["Kill it with fire?"],
			},
		},
	}

	if(db.Raid_Buffs) then
		local name = "|cff00FF33"..L["Raid_Buffs"].."|r"

		auraopts.args[db.label].args[name] = {
			order = 3,
			type = "group",
			inline = false,
			name = name,
			args = {},
		}

		local opts = auraopts.args[db.label].args[name]
		for k, v in next, db.Raid_Buffs do
			opts.args[k] = {
				order = 1,
				type = "group",
				inline = true,
				childGroups = "tab",
				name = k,
				args = {
					enabled = {
						order = 1,
						type = "toggle",
						name = L["Enable"],
						get = function(info) return db.Raid_Buffs[k].enabled end,
						set = function(info, val) db.Raid_Buffs[k].enabled = val; addon:LibreApply() end,
					},

					color = {
						order = 2,
						type = "color",
						name = L["Color"],
						get = function(info) return unpack(db.Raid_Buffs[k].color) end,
						set = function(info,r,g,b) db.Raid_Buffs[k].color = {r,g,b}; addon:LibreApply() end,
					},
				},
			}

		end
	end

	if(db.Survival_CDs) then
		local name = "|cffFFFF00"..L["Survival_CDs"].."|r"

		auraopts.args[db.label].args[name] = {
			order = 4,
			type = "group",
			inline = false,
			name = name,
			args = {},
		}

		local opts = auraopts.args[db.label].args[name]
		opts.args["color"] = {
			order = 0,
			type = "color",
			name = L["Border Color"],
			get = function(info) return unpack(db.Survival_CDs.color) end,
			set = function(info,r,g,b) db.Survival_CDs.color = {r,g,b}; addon:LibreApply() end,
		}

		opts.args["spacer"] = {
			order = 1,
			type = "description",
			name = "",
		}

		for k, v in next, db.Survival_CDs do
			if(k ~= "color") then
				opts.args[k] = {
					order = 2,
					type = "toggle",	
					name = k,
					get = function(info) return db.Survival_CDs[k] end,
					set = function(info, val) db.Survival_CDs[k] = val and true or false end,
				}
			end
		end
	end

	if(db.Boss_Debuffs) then
		local name = "|cffCC3300"..L["Boss_Debuffs"].."|r"

		auraopts.args[db.label].args[name] = {
			order = 5,
			type = "group",
			inline = false,
			name = name,
			args = {},
		}

		local opts = auraopts.args[db.label].args[name]
		opts.args["enabled"] = {
			order = 1,
			type = "toggle",
			name = L["Show Boss Debuffs"],
			get = function(info) return db.Boss_Debuffs.enabled end,
			set = function(info, val) db.Boss_Debuffs.enabled = val end,
		}

		opts.args["blacklist"] = {
			order = 2,
			type = "group",
			inline = true,
			name = L["Blacklist"],
			--disabled = function() return not db.Boss_Debuffs.enabled end,
			args = {
				name = {
					order = 1,
					type = "input",
					name = L["Name or Spell ID"],
					set = function(info, val)
						local spellid = tonumber(val)
						if(spellid) then
							val = addon.GetSpellInfo(val)
						end

						db.Boss_Debuffs[val] = true
					end,
					validate = function(info, val)
						local spellid = tonumber(val)
						if(spellid) then
							val = addon.GetSpellInfo(val)
						end

						if(db.Boss_Debuffs[val]) then
							return L["Boss Debuff: Already Exists"]
						elseif(val ~= "") then
							return true
						end
					end,
				},

				delete = {
					order = 2,
					type = "select",
					name = L["Delete"],
					set = function(info, val) db.Boss_Debuffs[val] = nil end,
					values = function(info)
						local bossdebuffs = {}
						for k, v in next, db.Boss_Debuffs do
							if(k ~= "enabled") then
								bossdebuffs[k] = k
							end
						end

						return bossdebuffs
					end,
					confirm = true,
					confirmText = L["Kill it with fire?"],
				},
			},
		}
	end

	return db
end

addon.options = {
	type = "group", name = ADDON_NAME, childGroups = "tab",
	args = {
		setup = setupopts,
		style = styleopts,
		aura = auraopts,
	},
}

local function toggleAnchors()
	local button = addon.options.args.anchors.args.lock

	if(addon.locked) then
		addon.locked = false
		button.name = L["Lock Anchors"]
	else
		addon.locked = true
		button.name = L["Unlock Anchors"]
	end

	addon:UpdateAnchors()
end

addon.options.args["anchors"] = {
	order = 0,
	type = "group",
	inline = true,
	name = "",
	args = {
		lock = {
			order = 0,
			type = "execute",
			name = L["Unlock Anchors"],
			func = function(info)
				toggleAnchors()
			end,
		},
	},
}

function addon:UpdateAuras()
	for k, v in next, addon.db.global.auraLists do		
		local db = addon:AddAuraList(k)

		for x, t in next, db.auras do
			addon:AddAura(db.label, db.auras, x)
		end
	end
end

local function ProfileChanged(event)	
	addon.profile = addon.db:GetCurrentProfile()	

	if(not addon.headers[addon.profile]) then
		addon.headers[addon.profile] = {}
	end

	if(not addon.anchors[addon.profile]) then
		addon.anchors[addon.profile] = {}
	end

	for k, v in next, addon.options.args.setup.args do 
		if(k ~= "create" and k ~= "anchors") then
			wipe(v)
		end
	end

	addon:CreateGroups()
	addon:UpdateAuras()

	if(InCombatLockdown()) then 
		addon.combatDelay:RegisterEvent("PLAYER_REGEN_ENABLED")

		if(event == "OnProfileReset") then
			addon.resetProfile[addon.profile] = true
		end
	else
		addon:UpdateHeaders()
		addon:UpdateAnchors()

		if(event == "OnProfileReset") then
			addon.headers[addon.profile] = {}
			addon.anchors[addon.profile] = {}
		end
	end
end

local function LoadConfig()
	if(addon.ConfigLoaded) then return end

	ProfileChanged()

	addon.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(addon.db)
	addon.options.args.profiles.order = 99

	LibStub("LibDualSpec-1.0"):EnhanceOptions(addon.options.args.profiles, addon.db)

	addon.options.name = ADDON_NAME.." - "..addon.version
	cfgreg:RegisterOptionsTable(ADDON_NAME.."_Config", addon.options)

	addon.db.RegisterCallback(addon, "OnProfileChanged", ProfileChanged)
	addon.db.RegisterCallback(addon, "OnProfileCopied", ProfileChanged)
	addon.db.RegisterCallback(addon, "OnProfileReset", ProfileChanged)

	addon.ConfigLoaded = true
end

_G["SLASH_".. ADDON_NAME:upper().."1"] = "/libre"
SlashCmdList[ADDON_NAME:upper()] = function(inp)
	local input = inp:match("%S+")
	if(input) then
		if(input == L["lock"] or input == L["unlock"]) then
			toggleAnchors()
		elseif(input == L["reset"]) then
			LibreDB = nil
			ReloadUI()
		end
	else
		LoadConfig()
		cfgdialog:Open(ADDON_NAME.."_Config")
	end
end

