local ADDON_NAME, addon = ...
local _VERSION = GetAddOnMetadata(ADDON_NAME, "version")

function addon.print(...)
	print("|cff00FF00"..ADDON_NAME..":|r", ...)
end

if(oUF and (oUF.version >= '1.6.6' and oUF.version < '1.6.99')) then
	if addon.oUF then wipe(addon.oUF) end
	addon.oUF = oUF
end
assert(addon.oUF, ADDON_NAME.." was unable to locate oUF install.")

local spellcache = setmetatable({}, 
{__index=function(t,id)
	if(GetSpellInfo(id)) then
		local info = {GetSpellInfo(id)}

		t[id] = info
		return t[id]
	end

	--addon.print("Couldn't get spell info: ", id)

	t[id] = {}
	return t[id]
end
})

addon.GetSpellInfo = function(id)
	if(not id or id == "") then return end
	return unpack(spellcache[id])
end

addon.SurvivalCooldowns = {
	color = {1,1,0},

	--[[ Tanks ]] --
	
	--DeathKnight
	[addon.GetSpellInfo(48792)] = true, --Icebound Fortitude	
	[addon.GetSpellInfo(55233)] = true, --Vampiric Blood

	--Druid
	[addon.GetSpellInfo(22812)] = true, --Barkskin
	[addon.GetSpellInfo(106922)] = true, --Might of Ursoc
	[addon.GetSpellInfo(61336)] = true, --Survival Instincts

	--Monk
	[addon.GetSpellInfo(115203)] = true, --Fortifying Brew
	[addon.GetSpellInfo(122278)] = true, --Dampen Harm

	--Paladin
	[addon.GetSpellInfo(642)] = true, --Divine Shield
	[addon.GetSpellInfo(31850)] = true, --Ardent Defender
	[addon.GetSpellInfo(86659)] = true, --Guardian of Ancient Kings
	
	--Warrior
	[addon.GetSpellInfo(871)] = true, --Shield Wall
	[addon.GetSpellInfo(12975)] = true, --Last Stand


	--[[ External ]] --
	
	--Druid
	[addon.GetSpellInfo(102342)] = true, --Ironbark

	--Monk
	[addon.GetSpellInfo(116849)] = true, --Life Cocoon

	--Paladin
	[addon.GetSpellInfo(1022)] = true, --Hand of Protection
	[addon.GetSpellInfo(6940)] = true, --Hand of Sacrifice

	--Priest
	[addon.GetSpellInfo(33206)] = true, --Pain Suppression
	[addon.GetSpellInfo(47788)] = true, --Guardian Spirit

	--Warrior
	[addon.GetSpellInfo(114030)] = true, --Vigilance
}

addon.RaidBuffs = {

	--DRUID,MONK,PALADIN
	["Stats"] = {
		enabled = false,
		color = {1,.5,0},
		spells = {
			[addon.GetSpellInfo(1126)] = true,--Mark of the Wild
			[addon.GetSpellInfo(115921)] = true,--Legacy of the Emperor
			[addon.GetSpellInfo(20217)] = true,--Blessing of Kings
		},
	},

	--PRIEST,WARLOCK,WARRIOR
	["Stamina"] = {
		enabled = false,
		color = {1,1,1},
		spells = {
			[addon.GetSpellInfo(21562)] = true,--Power Word: Fortitude
			[addon.GetSpellInfo(109773)] = true,--Dark Intent
			[addon.GetSpellInfo(469)] = true,--Commanding Shout
		},
	},

	--DEATHKNIGHT,HUNTER,WARRIOR
	["Attack Power"] = {
		enabled = false,
		color = {1,0,0},
		spells = {
			[addon.GetSpellInfo(57330)] = true,--Horn of Winter
			[addon.GetSpellInfo(19506)] = true,--Trueshot Aura
			[addon.GetSpellInfo(6673)] = true,--Battle Shout
		},
	},

	--MAGE,SHAMAN,WARLOCK
	["Spell Power"] = {
		enabled = false,
		color = {0,.5,1},
		spells = {
			[addon.GetSpellInfo(1459)] = true,--Arcane Brilliance
			[addon.GetSpellInfo(61316)] = true,--Dalaran Brilliance
			[addon.GetSpellInfo(77747)] = true,--Burning Wrath
			[addon.GetSpellInfo(109773)] = true,--Dark Intent
		},
	},

	--DEATHKNIGHT,ROGUE,SHAMAN
	["Haste"] = {
		enabled = false,
		color = {0,1,0},
		spells = {
			[addon.GetSpellInfo(55610)] = true,--Unholy Aura
			[addon.GetSpellInfo(113742)] = true,--Swiftblade's Cunning
			[addon.GetSpellInfo(30809)] = true,--Unleashed Rage
		},
	},

	--DRUID,PRIEST,SHAMAN
	["Spell Haste"] = {
		enabled = false,
		color = {.5,0,1},
		spells = {
			[addon.GetSpellInfo(24907)] = true,--Moonkin Aura
			[addon.GetSpellInfo(49868)] = true,--Mind Quickening
			[addon.GetSpellInfo(51470)] = true,--Elemental Oath	
		},
	},

	--DRUID,MAGE,MONK
	["Critical Strike"] = {
		enabled = false,
		color = {1,1,0},
		spells = {
			[addon.GetSpellInfo(17007)] = true,--Leader of the Pack
			[addon.GetSpellInfo(1459)] = true,--Arcane Brilliance
			[addon.GetSpellInfo(61316)] = true,--Dalaran Brilliance	
			[addon.GetSpellInfo(116781)] = true,--Legacy of the White Tiger
		},
	},	

	--PALADIN,SHAMAN
	["Mastery"] = {
		enabled = false,
		color = {0,1,1},
		spells = {
			[addon.GetSpellInfo(19740)] = true,--Blessing of Might
			[addon.GetSpellInfo(116956)] = true,--Grace of Air
		},
	},
}

local defaults = {
	global = {
		styles = {},
		auraLists = {},
	},

	profile = {
		groups = {},
		anchors = {},
	},
}

defaults.global.styles["default"] = {
	size = {
		w = 65,
		h = 40,
	},
	health = {
		statusbar = "",
		reverseClassColors = false,
	},
	power = {
		statusbar = "",
	},
	name = {	
		font = "",
		fontsize = 11,
		fontflag = ""
	},
	status = {
		color = {.7,.7,.7},
		font = "",
		fontsize = 13,
		fontflag = "",
	},
	cluster = {
		color = {0,1,0},
		font = "",
		fontsize = 12,
		fontflag = "",
	},
	heals = {
		maxOverflow = 1.05,
		myBar = {
			color = { 10/255, 202/255, 29/255, 0.5 },
		},
		otherBar = {
			color = { 2/255, 101/255, 18/255, 0.5 },
		},
		absorbBar = {
			color = { 200/255, 200/255, 0/255, 0.5 },
		},
		healAbsorbBar = {
			color = { 0/255, 0/255, 0/255, 0.3 },
		},
	},
	auras ={
		icons = {
			size = 14,
			bossSize = 18,
			num = 3,
			spacing = 6,
		},
		indys = {
			size = 8,
			num = 4,
			spacing = 2,
		}
	},
}

local function deepcopy(orig)
	local orig_type = type(orig)
	local copy
	if(orig_type == "table")then
		copy = {}
		for orig_key, orig_value in next, orig, nil do
			copy[deepcopy(orig_key)] = deepcopy(orig_value)
		end
		setmetatable(copy, deepcopy(getmetatable(orig)))
	else -- number, string, boolean, etc
		copy = orig
	end
	return copy
end
addon.deepcopy = deepcopy

local function StyleDBfunc(t, key)
	local val = defaults.global.styles.default[key]
	--print(key)

	if(val) then
		t[key] = deepcopy(val)
		return t[key]
	end
end

function addon:UpdateStyles()
	for k, v in next, addon.db.global.styles do
		v = setmetatable(v,{__index = StyleDBfunc})
	end
end

local function updateAuras()
	for k, v in next, addon.db.global.auraLists do
		-- update existing list with newly added CDs
		for x in next, addon.SurvivalCooldowns do
			if(v.Survival_CDs[x] == nil) then
				v.Survival_CDs[x] = true
			end	
		end

		-- remove old CDs
		for x in next, v.Survival_CDs do
			if(addon.SurvivalCooldowns[x] == nil) then
				v.Survival_CDs[x] = nil
			end
		end

		-- update Raid Buffs
		for x in next, v.Raid_Buffs do
			if(addon.RaidBuffs[x]) then
				v.Raid_Buffs[x].spells = {}

				for spell in next, addon.RaidBuffs[x].spells do
					v.Raid_Buffs[x].spells[spell] = true
				end
			else
				v.Raid_Buffs[x] = nil
			end
		end

		-- add missing Raid Buffs
		for x in next, addon.RaidBuffs do
			if(v.Raid_Buffs[x] == nil) then
				v.Raid_Buffs[x] = addon.deepcopy(addon.RaidBuffs[x])
			end
		end
	end
end

local addonFrame = CreateFrame"Frame"
addonFrame:RegisterEvent"ADDON_LOADED"
addonFrame:SetScript("OnEvent", function(self, event, arg1)
	if(arg1 ~= ADDON_NAME) then return end

	addon.db = LibStub("AceDB-3.0"):New("LibreDB", defaults)

	LibStub("LibDualSpec-1.0"):EnhanceDatabase(addon.db, ADDON_NAME)

	addon.version = _VERSION
	addon.profile = addon.db:GetCurrentProfile()
	addon.locked = true
	addon.headers[addon.profile] = {}
	addon.anchors[addon.profile] = {}
	addon.resetProfile = {}	

	updateAuras()
	addon:UpdateStyles()
	addon:CreateGroups()	

	self:UnregisterEvent"ADDON_LOADED"
end)

addon.anchors = {}
addon.headers = {}
addon.mods = {}

function addon:AddMod(Name, Update, Enable)
	addon.mods[Name] = {
		update = Update,
		enable = Enable,
	}
end

addon.SM = LibStub("LibSharedMedia-3.0", true)
addon.tags = addon.oUF.Tags.Methods
addon.tagevents = addon.oUF.Tags.Events
addon.class = select(2, UnitClass("player"))
addon.colors = setmetatable({
	power = setmetatable({
		["MANA"] = {.31,.45,.63},

	}, {__index = addon.oUF.colors.power}),

	--default hp color
	health = {49/255, 207/255, 37/255},

	debuff = setmetatable({},
	{__index = function(t,dtype)
		local color = DebuffTypeColor[dtype]
		if(color) then
			t[dtype] = {[1] = color.r, [2] = color.g, [3] = color.b}
			return t[dtype]
		else	
			return t.none
		end
	end})

}, {__index = addon.oUF.colors})

local format = string.format
function addon:hex(r, g, b)
	if(type(r) == 'table') then
		if(r.r) then r, g, b = r.r, r.g, r.b else r, g, b = unpack(r) end
	end
	return (r and ('|cff%02x%02x%02x'):format(r * 255, g * 255, b * 255)) or "|cffFFFFFF"
end

function addon:randomLabel(str)
	for i=1, 8 do
		local rand = math.random(0,1)
		if(rand == 1) then
			str = str..(("%d"):format(math.random(0,9)))
		else
			str = str..(string.char(math.random(65, 90)))
		end
	end

	return str
end

function addon:multicheck(check, ...)
	for i=1, select('#', ...) do
		if(check == select(i, ...)) then return true end
	end
	return false
end

function addon:validUnit(unit)
	local isValid = UnitExists(unit) and UnitIsConnected(unit) and UnitIsPlayer(unit) and
	not (UnitIsDeadOrGhost(unit) or UnitIsCharmed(unit) or UnitIsEnemy("player", unit))

	return isValid
end

addon.mediapath = "Interface\\AddOns\\Libre\\media\\"
addon.glowBorder = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = addon.mediapath.."glowTex", edgeSize = 4,
	insets = {left = 4, right = 4, top = 4, bottom = 4}
}

addon.Border = {
	bgFile = "Interface\\BUTTONS\\WHITE8X8",
	insets = {left = -1, right = -1, top = -1, bottom = -1}
}

local min, max, abs = math.min, math.max, abs
function addon:numberize(val)
	if(val >= 1e6) then
		return ("%.0fm"):format(val / 1e6)
	elseif(val >= 1e3) then
		return ("%.0fk"):format(val / 1e3)
	else
		return ("%d"):format(val)
	end
end

function addon:CreateAnchor(label)
	local name = ADDON_NAME..addon.profile..label.."Anchor"
	if(addon.anchors[addon.profile][name]) then 
		return addon.anchors[addon.profile][name]
	end

	local db = addon.db.profile.anchors[name]
	if(not db) then
		addon.db.profile.anchors[name] = {}

		db = addon.db.profile.anchors[name]
		db.point = "LEFT"
		db.x = 50
		db.y = 0
	end

	local OnDragStart = function(self)
		self:StartMoving()
	end

	local OnDragStop = function(self)
		self:StopMovingOrSizing()

		local point, relativeTo, relativePoint, xOffset, yOffset = self:GetPoint()	

		db.point = point
		db.x = xOffset
		db.y = yOffset
	end

	local anchor = CreateFrame("Frame", name, UIParent)
	anchor:SetFrameStrata"TOOLTIP"
	anchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background"})
	anchor:EnableMouse(true)
	anchor:SetMovable(true)
	anchor:SetClampedToScreen(true)
	anchor:RegisterForDrag"LeftButton"
	anchor:SetBackdropBorderColor(0, .9, 0)
	anchor:SetBackdropColor(0, .9, 0)
	anchor:SetSize(50,25)
	anchor:Hide()

	anchor:SetScript("OnDragStart", OnDragStart)
	anchor:SetScript("OnDragStop", OnDragStop)
	anchor:SetScript("OnHide", OnDragStop)

	anchor.text = anchor:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	anchor.text:SetPoint"CENTER"
	anchor.text:SetJustifyH"CENTER"
	anchor.text:SetFont(GameFontNormal:GetFont(), 12)
	anchor.text:SetTextColor(1, 1, 1)
	anchor.text:SetText(label)

	anchor.label = label
	addon.anchors[addon.profile][name] = anchor

	anchor:SetPoint(db.point, UIParent, db.point, db.x, db.y)

	return anchor
end

function addon:UpdateAnchors()
	for profile, t in next, addon.anchors do
		for name, anchor in next, t do
			if(profile == addon.profile) then
				local db = addon.db.profile.anchors[name]

				if(db and not addon.locked) then
					anchor:Show()
				else
					anchor:Hide()
				end
			else
				anchor:Hide()
			end
		end
	end
end

local function Smooth(self, value)
	if(value == self:GetValue()) then
		self.smoothing = nil
	else
		self.smoothing = value
	end
end

local GetFramerate = GetFramerate
local function SmoothUpdate(self)
	local value = self.smoothing
	if(not value) then return end

	local limit = 30/GetFramerate()
	local cur = self:GetValue()
	local new = cur + min((value-cur)/3, max(value-cur, limit))

	if(new ~= new) then
		new = value
	end

	self:SetValue_(new)
	if(cur == value or abs(new - value) < 2) then
		self:SetValue_(value)
		self.smoothing = nil
	end
end

addon.Smooth = Smooth
addon.SmoothUpdate = SmoothUpdate
addon.SetValue = CreateFrame("StatusBar").SetValue

