local _, addon = ...

local GetTime = GetTime
local format = string.format
local find = string.find

local function FormatTime(val)
	if(val > 60) then
		return format("%.0fm", (val/60)+0.5)
	else
		return format("%.0f", val)
	end
end

local function AuraTimer(self, elapsed)
	local timeLeft = (self.timeLeft or 0) - elapsed
	if(timeLeft <= 0) then
		self.time:SetText(nil)
		return
	end

	if(false) then
		self.time:SetText(FormatTime(timeLeft))
	else
		self.time:SetText(nil)
	end

	self.timeLeft = timeLeft
end

local function getIcon(icons)
	local n = icons.visible + 1
	if(icons[n]) then
		return icons[n]
	end

	icons.created = icons.created + 1

	local button = CreateFrame("Button", nil, icons)
	button:EnableMouse(false)

	local icon = button:CreateTexture(nil, "BORDER")
	icon:SetAllPoints(button)
	icon:SetTexCoord(.07, .93, .07, .93)

	local cd = CreateFrame("Cooldown", nil, button)
	cd:SetAllPoints(button)
	cd.noCooldownCount = true
	cd:SetReverse(true)

	local border = CreateFrame("Frame", nil, button)
	border:SetPoint("TOPLEFT", -1, 1)
	border:SetPoint("BOTTOMRIGHT", 1, -1)	
	border:SetFrameLevel(button:GetFrameLevel()-1)
	border:SetBackdrop(addon.Border)
	border:SetBackdropColor(0,0,0,1)
	border:SetBackdropBorderColor(0,0,0,1)

	local count = cd:CreateFontString(nil, "OVERLAY")
	count:SetFontObject(NumberFontNormalSmall)
	count:SetTextColor(1,1,0)
	count:SetPoint("BOTTOMRIGHT", 6, -5)

	local time = cd:CreateFontString(nil, "OVERLAY")
	time:SetFontObject(NumberFontNormalSmall)
	time:SetTextColor(1,1,1)
	time:SetPoint("TOPLEFT", -5, 6)

	table.insert(icons, button)

	button.icon = icon
	button.cd = cd
	button.border = border
	button.count = count
	button.time = time
	button:SetScript("OnUpdate", AuraTimer)

	return button
end

local indyTex = "Interface\\BUTTONS\\WHITE8X8"
local function getIndy(indys)
	local n = indys.visible + 1
	if(indys[n]) then
		return indys[n]
	end

	indys.created = indys.created + 1

	local button = CreateFrame("Button", nil, indys)
	button:EnableMouse(false)

	local icon = button:CreateTexture(nil, "BORDER")
	icon:SetAllPoints(button)
	icon:SetTexture(indyTex)

	local cd = CreateFrame("Cooldown", nil, button)
	cd:SetAllPoints(button)
	cd.noCooldownCount = true
	cd:SetReverse(true)	

	local border = CreateFrame("Frame", nil, button)
	border:SetPoint("TOPLEFT", -4, 4)
	border:SetPoint("BOTTOMRIGHT", 4, -4)
	border:SetFrameLevel(button:GetFrameLevel()-1)
	border:SetBackdrop(addon.glowBorder)
	border:SetBackdropColor(0,0,0,1)
	border:SetBackdropBorderColor(0,0,0,1)

	table.insert(indys, button)

	button.icon = icon
	button.cd = cd
	button.border = border

	return button
end

local function setAura(auras, aura, duration, timeLeft)
	local cd = aura.cd
	if(cd) then
		if(not aura.disableCD and duration and duration > 0) then
			cd:SetCooldown(timeLeft - duration, duration)
			cd:Show()
		else
			cd:Hide()
		end
	end

	if(aura.isBossDebuff) then
		aura:SetSize(auras.bossSize, auras.bossSize)
	else
		aura:SetSize(auras.size, auras.size)
	end
	aura:Show()

	auras.visible = auras.visible + 1
end

local function iconFilter(icons, icon, isPlayer, isDebuff, ...)
	local name, texture, count, dtype, duration, timeLeft, caster, spellID, isBossDebuff = ...

	local spell = icons.list.auras[name]

	local show, color
	if(spell and spell.icons) then
		if(spell.player and not isPlayer) then
			return false
		elseif(spell.spellid and spell.spellid ~= spellID) then
			return false
		end

		show = true
		color = isDebuff and (addon.colors.debuff[dtype])
	elseif(isBossDebuff) then
		local bossdb = icons.list.Boss_Debuffs
		if(bossdb.enabled and not bossdb[name]) then
			show = true
			color = isDebuff and (addon.colors.debuff[dtype])
		end
	elseif(not isDebuff and icons.list.Survival_CDs[name]) then
		show = true
		color = icons.list.Survival_CDs.color
	elseif(isDebuff and icons.list[dtype]) then
		show = true
		color = isDebuff and (addon.colors.debuff[dtype])
	end

	if(show) then
		if(icon.time) then
			icon.duration = duration
			icon.timeLeft = timeLeft - GetTime()
		end	

		icon.icon:SetTexture(texture)
		icon.count:SetText((count > 1 and count))

		if(color) then
			icon.border:SetBackdropColor(unpack(color))
		else	
			icon.border:SetBackdropColor(0,0,0,0)
		end

		setAura(icons, icon, duration, timeLeft)
	end
end

local function indyFilter(indys, indy, isPlayer, isDebuff, ...)
	local name, texture, count, dtype, duration, timeLeft, caster, spellID, isBossDebuff = ...

	local spell = indys.list.auras[name]

	local show
	if(spell and spell.indicators) then
		if((spell.player and not isPlayer)) then
			return false
		elseif(spell.spellid and spell.spellid ~= spellID) then
			return false
		end

		indy.disableCD = spell.disableCD
		indy.color = spell.color

		show = true
	end

	if(show) then
		indy.icon:SetVertexColor(unpack(indy.color))

		setAura(indys, indy, duration, timeLeft)
	end
end

local function scanAuras(unit, icons, indys, index, filter, isDebuff)
	local name, rank, texture, count, dtype, duration, timeLeft, caster, isStealable,
	shouldConsolidate, spellID, canApplyAura, isBossDebuff = UnitAura(unit, index, filter)

	if(name) then
		local icon = getIcon(icons)
		local indy = getIndy(indys)

		local isPlayer
		if(caster == "player" or caster == "vehicle") then
			isPlayer = true
		end

		icon.isBossDebuff = isBossDebuff

		iconFilter(icons, icon, isPlayer, isDebuff, name, texture, count, dtype, duration, timeLeft, caster, spellID, isBossDebuff)
		indyFilter(indys, indy, isPlayer, isDebuff, name, texture, count, dtype, duration, timeLeft, caster, spellID, isBossDebuff)

		return true
	end
end

local function scanBuffs(unit, indys, buff)
	if(buff.enabled) then
		local active
		for name in next, buff.spells do
			active = UnitBuff(unit, name)
			if(active) then break end
		end

		if(not active) then
			local indy = getIndy(indys)
			indy.color = buff.color
			indy.icon:SetVertexColor(unpack(indy.color))

			setAura(indys, indy)
		end
	end
end

local function SetPosition(auras, from, to)	
	local anchor = auras.initialAnchor or "BOTTOMLEFT"
	local spacing = (auras['spacing-x'] or auras.spacing or 0)

	local point
	if(anchor:find("RIGHT")) then
		point = (anchor:find("TOP") and "TOPLEFT") or "BOTTOMLEFT"
		spacing = -spacing
	else
		point = (anchor:find("TOP") and "TOPRIGHT") or "BOTTOMRIGHT"
	end

	for i = from, to do
		local button = auras[i]
		if(not button) then break end

		button:ClearAllPoints()	
		if(i == 1) then
			button:SetPoint(anchor, auras, anchor)
		else
			button:SetPoint(anchor, auras[i-1], point, spacing, 0)
		end
	end
end

local function finalize(auras)
	local visible = auras.visible

	if(auras.created > auras.anchored) then
		SetPosition(auras, auras.anchored + 1, auras.created)
		auras.anchored = auras.created
	end

	local max = auras.max
	visible = (visible < max and visible) or max

	for i=visible+1, #auras do
		auras[i]:Hide()
	end
end

local function UpdateAuras(self, event, unit)
	if(self.unit ~= unit) then return end

	local icons = self.libreIcons
	local indys = self.libreIndicators

	if(not addon:validUnit(unit)) then
		icons:Hide()
		indys:Hide()
	else
		icons:Show()
		indys:Show()
	end	

	icons.visible = 0
	indys.visible = 0

	local index = 1
	while(true) do
		local result = scanAuras(unit, icons, indys, index, "HARMFUL", true)
		if(not result) then break end

		index = index + 1
	end

	index = 1
	while(true) do
		local result = scanAuras(unit, icons, indys, index)
		if(not result) then break end

		index = index + 1
	end

	local raidBuffs = indys.list.Raid_Buffs
	if(raidBuffs) then
		for _, buff in next, raidBuffs do
			scanBuffs(unit, indys, buff)
		end
	end

	finalize(icons)
	finalize(indys)
end

local dummyList = {
	Raid_Buffs = {},
	Survival_CDs = {},
	Boss_Debuffs = {},
	auras = {},
}

local Update = function(self)
	local auraList = addon.db.global.auraLists[self:GetParent().db.auras]

	local icons = self.libreIcons

	icons:ClearAllPoints()
	icons:SetPoint("BOTTOMLEFT", self.Overlay, "BOTTOMLEFT", 2, 2)
	icons.size = self.db.auras.icons.size or 14
	icons.bossSize = self.db.auras.icons.bossSize or 18
	icons.max = self.db.auras.icons.num or 3
	icons.spacing = self.db.auras.icons.spacing or 6
	icons:SetSize(self:GetWidth(), icons.size)
	icons.initialAnchor = "BOTTOMLEFT"
	icons['growth-x'] = "RIGHT"
	icons.list = auraList or dummyList

	local indicators = self.libreIndicators

	indicators:ClearAllPoints()
	indicators:SetPoint("TOPRIGHT", self.Overlay, "TOPRIGHT", -1, -1)
	indicators.size = self.db.auras.indys.size or 8
	indicators.max = self.db.auras.indys.num or 4
	indicators.spacing = self.db.auras.indys.spacing or 2
	indicators:SetSize(self:GetWidth(), indicators.size)
	indicators.initialAnchor = "TOPRIGHT"
	indicators['growth-x'] = "LEFT"
	indicators.list = auraList or dummyList

	UpdateAuras(self, nil, self.unit)
end

local Enable = function(self)
	local icons = CreateFrame("Frame", nil, self.Overlay)
	icons.created = 0
	icons.anchored = 0

	self.libreIcons = icons

	local indicators = CreateFrame("Frame", nil, self.Overlay)
	indicators.created = 0
	indicators.anchored = 0

	self.libreIndicators = indicators

	self:RegisterEvent("UNIT_AURA", UpdateAuras)
	self:RegisterEvent("UNIT_MAXHEALTH", UpdateAuras)
	self:RegisterEvent("UNIT_CONNECTION", UpdateAuras)
end

addon:AddMod("Aura", Update, Enable)

