local ADDON_NAME, addon = ...

local MapData = LibStub("LibMapData-1.0")
local GetPlayerMapPosition = GetPlayerMapPosition
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax

local mapwidth, mapheight = 0, 0
local units = {}

addon.tags[ADDON_NAME..":cluster"] = function(unit)
	if(units[unit]) then
		local num = units[unit].numInRange	

		if(num > 6) then
			return "6+"
		elseif(num > 2) then
			return num
		end

		--return num
	end
end

local function getDistance(x1, y1, x2, y2)
	local xx = (x2 - x1) * mapwidth
	local yy = (y2 - y1) * mapheight

	return (xx*xx + yy*yy)^0.5
end

local function clearDataid(id, id2)
	if(not id2) then
		if(units[id].numInRange > 1) then
			units[id].cluster = {}
			units[id].numInRange = 1
		end

		for id2 in next, units do
			if(units[id2].cluster[id]) then
				units[id2].cluster[id] = nil
				units[id2].numInRange = units[id2].numInRange-1
			end
		end
	else
		if(units[id].cluster[id2]) then
			units[id].cluster[id2] = nil
			units[id].numInRange = units[id].numInRange-1
		end
		if(units[id2].cluster[id]) then
			units[id2].cluster[id] = nil
			units[id2].numInRange = units[id2].numInRange-1
		end
	end
end

local function updateCluster(self, elapsed)
	self.elapsed = (self.elapsed or 0) + elapsed
	if(self.elapsed < self.update) then return end
	
	self.elapsed = 0
	--print(GetTime(), " | ", self.range, " | ", self.update, " | ", self.minHP)

	for id in next, units do
		if(addon:validUnit(id)) then
			local x, y = GetPlayerMapPosition(id)
			units[id].pos.x = x
			units[id].pos.y = y

			local cur, max = UnitHealth(id), UnitHealthMax(id)
			if(max > 0) then
				local per = cur/max
				units[id].hp = per
			else
				units[id].hp = 0
			end
		end
	end

	for id in next, units do
		local hp = units[id].hp

		if(hp > 0 and addon:validUnit(id)) then
			local pos = units[id].pos
			
			for id2 in next, units do
				local hp2 = units[id2].hp

				if(hp2 > 0 and addon:validUnit(id2) and (id ~= id2)) then
					local pos2 = units[id2].pos
					local x1, y1 = pos.x, pos.y
					local x2, y2 = pos2.x, pos2.y
					local xxyy = x1 + x2 + y1 + y2

					local dist = getDistance(x1, y1, x2, y2)
					if(xxyy > 0 and dist <= self.range and (hp < self.minHP and hp2 < self.minHP)) then
						if(not units[id].cluster[id2]) then
							units[id].cluster[id2] = true
							units[id].numInRange = units[id].numInRange+1
						end
						if(not units[id2].cluster[id]) then
							units[id2].cluster[id] = true
							units[id2].numInRange = units[id2].numInRange+1	
						end
					else
						clearDataid(id, id2)
					end
				end
			end
		else
			clearDataid(id)
		end
	end
end

local px, py = 0, 0
local function updateMapFiles()
	px, py = GetPlayerMapPosition("player")

	if((px or 0)+(py or 0) <= 0) then
		SetMapToCurrentZone()
		px, py = GetPlayerMapPosition("player")
		if((px or 0)+(py or 0) <= 0) then return end
	end

	if(px > 0 or py > 0) then
		local map, level = GetMapInfo(), GetCurrentMapDungeonLevel()
		mapwidth, mapheight = MapData:MapArea(map, level)
	end
end

local function fillroster(unit)
	units[unit] = { 
		["pos"] = {},
		["numInRange"] = 1,
		["hp"] = 0,
		["cluster"] = {},
	}
end

local function updateRoster()
	wipe(units)

	if(IsInRaid()) then
		local numMembers = GetNumGroupMembers()

		for i=1, numMembers do
			local unit = "raid"..i
			fillroster(unit)
		end
	--elseif(IsInGroup(LE_PARTY_CATEGORY_INSTANCE)) then
	else
		local numMembers = GetNumSubgroupMembers()

		fillroster("player")
		for i=1, numMembers do
			local unit = "party"..i
			fillroster(unit)
		end
	end
end

local _Frame = CreateFrame"Frame"
_Frame:SetScript("OnEvent", function(self, event)
	if(event == "ZONE_CHANGED_NEW_AREA" or event == "PLAYER_ENTERING_WORLD") then
		--print("zoneevent")
		updateMapFiles()
	end

	if(event == "GROUP_ROSTER_UPDATE") then
		--print("rosterevent")
		updateRoster()
	end
end)

local Update = function(self)	
	_Frame.update = .5
	_Frame.range = 15
	_Frame.minHP = 90 / 100

	self.Cluster:SetFont(addon.SM:Fetch("font", self.db.cluster.font),
	self.db.cluster.fontsize, self.db.cluster.fontflag)

	self.Cluster:SetTextColor(unpack(self.db.cluster.color))
	self.Cluster.frequentUpdates = _Frame.update
	self:Tag(self.Cluster, "["..ADDON_NAME..":cluster]")

	updateRoster()
end

local Enable = function(self)
	self.Cluster = self.Overlay:CreateFontString(nil, "OVERLAY")
	self.Cluster:SetShadowOffset(1.25, -1.25)
	self.Cluster:SetPoint("BOTTOMRIGHT", -4, 1)
	self.Cluster:SetJustifyH("RIGHT")

	_Frame:RegisterEvent("GROUP_ROSTER_UPDATE")
	_Frame:RegisterEvent("PLAYER_ENTERING_WORLD")
	_Frame:RegisterEvent("ZONE_CHANGED_NEW_AREA")

	_Frame:SetScript("OnUpdate", updateCluster)
	_Frame.update = .5
	_Frame.range = 15
	_Frame.minHP = 90 / 100
end

addon:AddMod("Cluster", Update, Enable)

