local ADDON_NAME, addon = ...

local cache = {}
local function getName(unit, name)
	local GUID = UnitGUID(unit)
	if(cache[GUID]) then
		return cache[GUID]
	end

	local _, class = UnitClass(unit)
	if(class) then
		local color = addon.colors.class[class]
		cache[GUID] = addon:hex(color)..name

		return cache[GUID]
	end
end

addon.tags[ADDON_NAME..":name"] = function(unit, real)
	local name = GetUnitName((real or unit), false)
	return name
end
addon.tagevents[ADDON_NAME..":name"] = addon.tagevents.name

addon.tags[ADDON_NAME..":colorname"] = function(unit, real)
	local name = GetUnitName((real or unit), false)

	if(name) then
		return getName(unit, name)
	end
end
addon.tagevents[ADDON_NAME..":colorname"] = addon.tagevents.name

addon.tags[ADDON_NAME..":deadcolorname"] = function(unit, real)
	local name = GetUnitName((real or unit), false)

	if(name) then
		if(UnitIsDeadOrGhost(unit)) then
			return getName(unit, name)
		end

		return name
	end
end
addon.tagevents[ADDON_NAME..":deadcolorname"] = addon.tagevents.name.." UNIT_MAXHEALTH UNIT_HEALTH"

local roles = {
	["HEALER"] = true,
	["TANK"] = true,
	["DAMAGER"] = false,
}

local roleOverride = function(self, event)
	local lfdrole = self.LFDRole
	local role = UnitGroupRolesAssigned(self.unit)
	local nameHeight = self.Name:GetHeight()

	if(roles[role]) then
		lfdrole:SetTexCoord(GetTexCoordsForRoleSmallCircle(role))
		lfdrole:SetSize(nameHeight, nameHeight)

		lfdrole:Show()
	else	
		lfdrole:SetSize(1, nameHeight)

		lfdrole:Hide()
	end
end

local Update = function(self)
	if(self.LFDRole:IsShown()) then
		self.LFDRole:SetSize(self.db.name.fontsize, self.db.name.fontsize)
	end

	self.Name:SetFont(addon.SM:Fetch("font", self.db.name.font),
	self.db.name.fontsize, self.db.name.fontflag)

	if(true) then
		self:Tag(self.Name, "["..ADDON_NAME..":deadcolorname]")
	elseif(false) then
		self:Tag(self.Name, "["..ADDON_NAME..":colorname]")
	else
		self:Tag(self.Name, "["..ADDON_NAME..":name]")
	end
end

local Enable = function(self)
	local role = self.Overlay:CreateTexture(nil, "OVERLAY")
	role:SetTexture[[Interface\LFGFrame\UI-LFG-ICON-PORTRAITROLES]]
	role:SetPoint("TOPLEFT", 1, -1)	
	role.Override = roleOverride

	self.LFDRole = role

	local name = self.Overlay:CreateFontString(nil, "OVERLAY")
	name:SetShadowOffset(1.25, -1.25)
	name:SetPoint("TOPLEFT", role, "TOPRIGHT", 0, 0)
	name:SetPoint("TOPRIGHT", -3, 0)
	name:SetJustifyH"LEFT"
	name.overrideUnit = true

	self.Name = name
end

addon:AddMod("Name", Update, Enable)

