local _, addon = ...

local GetDisplayPower = function(power, unit)
	local barType = UnitAlternatePowerInfo(unit)
	if(power.displayAltPower and barType) then
		power.isAlt = true
		return ALTERNATE_POWER_INDEX
	else
		power.isAlt = false
		return UnitPowerType(unit)
	end
end

local tmpName
local function OverridePower(self, event, unit)
	if(self.unit ~= unit) then return end
	local power = self.Power

	local displayType = GetDisplayPower(power, unit)
	local min, max = UnitPower(unit, displayType), UnitPowerMax(unit, displayType)
	local disconnected = not UnitIsConnected(unit)
	power:SetMinMaxValues(0, max)

	if(disconnected) then
		power:SetValue(0)
	else
		power:SetValue(min)
	end

	power.disconnected = disconnected

	local tPath, r, g, b, t
	if(power.isAlt) then
		local powerName = select(10, UnitAlternatePowerInfo(unit))

		if(self.colors.power[powerName]) then
			t = self.colors.power[ptoken]
		else
			tPath, r, g, b = UnitAlternatePowerTextureInfo(unit, 2)
		end
		
		if(powerName and powerName ~= tmpName) then
			--print(powerName)
			tmpName = powerName
		end
	else
		local ptype, ptoken, altR, altG, altB = UnitPowerType(unit)

		if(ptoken == "MANA") then
			t = self.colors.power[ptoken]
		end
	end

	if(t) then
		r, g, b = t[1], t[2], t[3]
	end

	if(b) then
		power:SetStatusBarColor(r, g, b)

		local bg = power.bg
		if(bg) then
			local mu = bg.multiplier or 1
			bg:SetVertexColor(r * mu, g * mu, b * mu)
		end

		local pHeight = .10
		power:SetHeight(self:GetHeight()*pHeight)
		self.Health:SetHeight(self:GetHeight()*(.99-pHeight))
	else
		power:SetHeight(.001)
		self.Health:SetHeight(self:GetHeight())
	end
end

local Update = function(self)
	self.Power:SetStatusBarTexture(addon.SM:Fetch("statusbar", self.db.power.statusbar), "BORDER")
	self.Power:ClearAllPoints()
	self.Power:SetPoint("BOTTOM")
	self.Power:SetPoint("LEFT")
	self.Power:SetPoint("RIGHT")
	self.Power.displayAltPower = true

	self.Power.bg:SetTexture(addon.SM:Fetch("statusbar", self.db.power.statusbar))
	self.Power.bg.multiplier = .2
end

local Enable = function(self)
	local pp = CreateFrame("StatusBar", nil, self)
	pp.Override = OverridePower
	pp.SetValue_ = addon.SetValue 
	pp.SetValue = addon.Smooth
	pp:SetScript("OnUpdate", addon.SmoothUpdate)

	self.Power = pp

	local ppbg = pp:CreateTexture(nil, "BACKGROUND")
	ppbg:SetAllPoints(pp)

	self.Power.bg = ppbg
end

addon:AddMod("Power", Update, Enable)

