-- Modified oUF Range and Arrow functions from TomTom

local _, addon = ...

local inAlpha, outAlpha = 1, .4
local _FRAMES = {}
local OnRangeFrame

local UnitInRange, UnitIsConnected = UnitInRange, UnitIsConnected
local SetMapToCurrentZone, WorldMapFrame = SetMapToCurrentZone, WorldMapFrame
local GetPlayerMapPosition, GetPlayerFacing = GetPlayerMapPosition, GetPlayerFacing

local select, next = select, next
local pi = math.pi
local twopi = pi * 2
local atan2 = math.atan2
local modf = math.modf
local abs = math.abs
local floor = floor

local function ColorGradient(perc, ...)
	local num = select("#", ...)
	local hexes = type(select(1, ...)) == "string"

	if(perc == 1) then
		return select(num-2, ...), select(num-1, ...), select(num, ...)
	end

	num = num / 3

	local segment, relperc = modf(perc*(num-1))
	local r1, g1, b1, r2, g2, b2
	r1, g1, b1 = select((segment*3)+1, ...), select((segment*3)+2, ...), select((segment*3)+3, ...)
	r2, g2, b2 = select((segment*3)+4, ...), select((segment*3)+5, ...), select((segment*3)+6, ...)

	if(not r2 or not g2 or not b2) then
		return r1, g1, b1
	else
		return r1 + (r2-r1)*relperc,
		g1 + (g2-g1)*relperc,
		b1 + (b2-b1)*relperc
	end
end

local function ColorTexture(texture, angle)
	local perc = abs((pi - abs(angle)) / pi)

	local gr,gg,gb = 0, 1, 0
	local mr,mg,mb = 1, 1, 0
	local br,bg,bb = 1, 0, 0
	local r,g,b = ColorGradient(perc, br, bg, bb, mr, mg, mb, gr, gg, gb)

	texture:SetVertexColor(r,g,b)
end

local function RotateTexture(self, angle)
	angle = angle - GetPlayerFacing()

	local cell = floor(angle / twopi * 108 + 0.5) % 108
	if(cell == self.cell) then return end
	self.cell = cell

	local column = cell % 9
	local row = floor(cell / 9)

	ColorTexture(self.arrow, angle)
	local xstart = (column * 56) / 512
	local ystart = (row * 42) / 512
	local xend = ((column + 1) * 56) / 512
	local yend = ((row + 1) * 42) / 512
	self.arrow:SetTexCoord(xstart,xend,ystart,yend)
end

local px, py, tx, ty
local function GetBearing(unit)
	if(unit == "player" or WorldMapFrame:IsShown()) then return end

	px, py = GetPlayerMapPosition("player")
	if((px or 0)+(py or 0) <= 0) then
		SetMapToCurrentZone()
		px, py = GetPlayerMapPosition("player")
		if((px or 0)+(py or 0) <= 0) then return end
	end

	tx, ty = GetPlayerMapPosition(unit)
	if((tx or 0)+(ty or 0) <= 0) then return end

	return pi - atan2(px-tx,ty-py)
end

local timer = 0
local function RangeUpdate(self, elapsed)
	timer = timer + elapsed
	if(timer < .15) then return end

	timer = 0
	for _, object in next, _FRAMES do
		if(object:IsShown() and object:IsVisible()) then
			if(UnitIsConnected(object.unit)) then
				local inRange, checkedRange = UnitInRange(object.unit)
				if(checkedRange and not inRange) then
					object.OoR = true
					object:SetAlpha(outAlpha)

					local bearing = GetBearing(object.unit)
					if(bearing) then
						RotateTexture(object, bearing)
						object.arrowFrame:Show()
					else
						object.arrowFrame:Hide()
					end
				else
					object.OoR = false
					object:SetAlpha(inAlpha)
					object.arrowFrame:Hide()
				end
			end
		else
			object.OoR = false
			object.arrowFrame:Hide()
		end
	end
end

local function Update(self)
	self.arrowFrame:SetScale(.05)
end

local Enable = function(self)
	self.arrowFrame = CreateFrame("Frame", nil, UIParent)
	self.arrowFrame:SetAllPoints(self)
	self.arrowFrame:SetFrameLevel(self.Overlay:GetFrameLevel()+1)
	self.arrowFrame:Hide()

	self.arrow = self.arrowFrame:CreateTexture(nil, "OVERLAY")
	self.arrow:SetTexture(addon.mediapath.."Arrow")
	self.arrow:SetPoint("RIGHT", self, "RIGHT")

	table.insert(_FRAMES, self)

	if(not OnRangeFrame) then
		OnRangeFrame = CreateFrame"Frame"
		OnRangeFrame:SetScript("OnUpdate", RangeUpdate)
	end
	OnRangeFrame:Show()
end

addon:AddMod("Range", Update, Enable)
